@@ -1,4 +1,4 @@
-#!/usr/local/bin/python2.6
+#!/usr/local/bin/python2.7
 # -*- coding: utf-8 -*-
 # -----------------------------------------------------------------------------
 # Gettings Things Gnome! - a personal organizer for the GNOME desktop
@@ -37,27 +37,26 @@
 #
 #   Each id are *strings*
 #   tid are the form "X@Y" where Y is the pid.
-#   For example : 21@2 is the 21th task of the 2nd project
+#   For example : 21@2 is the 21st task of the 2nd project
 #   This way, we are sure that a tid is unique accross multiple projects
 #
 #==============================================================================
 
-#=== IMPORT ===================================================================
-from __future__ import with_statement
+"""This is the top-level exec script for running GTG"""
 
-import sys
+#=== IMPORT ===================================================================
 import os
-import dbus
+import sys
 import logging
-import signal
-from contextlib import contextmanager
+import dbus
 
 #our own imports
-from GTG import _
-from GTG.taskbrowser.browser import TaskBrowser
+from GTG.backends       import BackendFactory
+from GTG                import _
+from GTG.core           import CoreConfig
 from GTG.core.datastore import DataStore
-from GTG.core.dbuswrapper import DBusTaskWrapper
-from GTG.core import CoreConfig
+from GTG.gtk.manager    import Manager
+from GTG.tools.logger   import Log
 
 #=== OBJECTS ==================================================================
 
@@ -66,8 +65,11 @@ from GTG.core import CoreConfig
 #that's why we put the pid file in the data directory :
 #we allow one instance of gtg by data directory.
 
-def check_instance(directory):
-    """Check if gtg is already running."""
+def check_instance(directory, uri_list = []):
+    """
+    Check if gtg is already running.
+    If so, open the tasks whose ids are in the uri_list
+    """
     pidfile = os.path.join(directory, "gtg.pid")
     if not os.path.exists(pidfile):
         open(pidfile, "w").close()
@@ -80,67 +82,91 @@ def check_instance(directory):
         p_name = os.popen("/bin/ps -p %s" % pid).read()
         if p == 0 and "gtg" in p_name:
             print _("gtg is already running!")
-            d=dbus.SessionBus().get_object(CoreConfig.BUSNAME,\
+            try:
+                d=dbus.SessionBus().get_object(CoreConfig.BUSNAME,\
                                            CoreConfig.BUSINTERFACE)
-            d.show_task_browser()
-            sys.exit(0)
-            
+                d.ShowTaskBrowser()
+                #if the user has specified a task to open, do that
+                for uri in uri_list:
+                    if uri.startswith("gtg://"):
+                        d.OpenTaskEditor(uri[6:])
+                raise SystemExit
+            except dbus.exceptions.DBusException:
+                # If we cant't connect to the interface (e.g. changed interface
+                # between GTG versions), we won't do anything more
+                raise SystemExit
+
     #write the pid file
     with open(pidfile, "w") as f:
         f.write(`os.getpid()`)
 
+def remove_pidfile(directory):
+    """Remove the pid file"""
+    pidfile = os.path.join(directory, "gtg.pid")
+    os.remove(pidfile)
+
 #=== MAIN CLASS ===============================================================
 
 def main(options=None, args=None):
-    
-    # init logging system
-    logger = logging.getLogger("gtg_logger")
-    ch = logging.StreamHandler()
-    ch.setLevel(logging.DEBUG)
-    formatter = logging.Formatter("%(asctime)s - %(levelname)s - %(module)s:%(funcName)s:%(lineno)d - %(message)s")
-    ch.setFormatter(formatter)
-    logger.addHandler(ch)
+    '''
+    Calling this starts the full GTG experience  ( :-D )
+    '''
+    ds, req = core_main_init(options, args)
+    # Launch task browser
+    manager = Manager(req)
+    #main loop
+    #To be more user friendly and get the logs of crashes, we show an apport
+    # hooked window upon crashes
+    if options.no_crash_handler == False:
+        from GTG.gtk.crashhandler import signal_catcher
+        with signal_catcher(manager.close_browser):
+            manager.main(once_thru=options.boot_test, uri_list = args)
+    else:
+        manager.main(once_thru=options.boot_test, uri_list = args)
+    core_main_quit(req, ds)
+
+def core_main_init(options = None, args = None):
+    '''
+    Part of the main function prior to the UI initialization.
+    '''
+    # Debugging subsystem initialization
     if options.debug:
-        logger.setLevel(logging.DEBUG)
-        logger.debug("Debug output enabled.")
-    
+        Log.setLevel(logging.DEBUG)
+        Log.debug("Debug output enabled.")
+    else:
+        Log.setLevel(logging.INFO)
+    Log.set_debugging_mode(options.debug)
     config = CoreConfig()
-    check_instance(config.DATA_DIR)
-    backends_list = config.get_backends_list()
-    
+    check_instance(config.get_data_dir(), args)
+    backends_list = BackendFactory().get_saved_backends_list()
     # Load data store
-    ds = DataStore()
-    
+    ds = DataStore(config)
+    # Register backends
     for backend_dic in backends_list:
         ds.register_backend(backend_dic)
-    
-    #save directly the backends to be sure to write projects.xml
-    config.save_datastore(ds)
-        
+    #save the backends directly to be sure projects.xml is written
+    ds.save(quit = False)
+
     # Launch task browser
     req = ds.get_requester()
-    tb = TaskBrowser(req, config, logger=logger)
-    DBusTaskWrapper(req, tb)
-    #we listen for signals from the system in order to save our configuration
-    # if GTG is forcefully terminated (e.g.: on shutdown).
-    @contextmanager
-    def signal_catcher():
-        #if TERM or ABORTare caught, we close the browser
-        for s in [signal.SIGABRT, signal.SIGTERM]:
-            signal.signal(s, lambda a,b: tb.on_close())
-        yield
+    return ds, req
 
-    #main loop
-    with signal_catcher():
-        tb.main()
-
-    # Ideally we should load window geometry configuration from a config.
+def core_main_quit(req, ds):
+    '''
+    Last bits of code executed in GTG, after the UI has been shut off.
+    Currently, it's just saving everything.
+    '''
+    # Ideally we should load window geometry configuration from a config
     # backend like gconf at some point, and restore the appearance of the
     # application as the user last exited it.
-
+    #
     # Ending the application: we save configuration
-    config.save_config()
-    config.save_datastore(ds)
+    req.save_config()
+    ds.save(quit = True)
+    config = req.get_global_config()
+    remove_pidfile(config.get_data_dir())
+    sys.exit(0)
+
 
 #=== EXECUTION ================================================================
 
