@@ -20,6 +20,7 @@
 #Functions to convert a Task object to an XML string and back
 import xml.dom.minidom
 import xml.sax.saxutils as saxutils
+import datetime
 
 from GTG.tools import cleanxml
 from GTG.tools import dates
@@ -38,7 +39,7 @@ def task_from_xml(task,xmlnode) :
     sub_list = xmlnode.getElementsByTagName("subtask")
     for s in sub_list :
         sub_tid = s.childNodes[0].nodeValue
-        cur_task.add_subtask(sub_tid)
+        cur_task.add_child(sub_tid)
     attr_list = xmlnode.getElementsByTagName("attribute")
     for a in attr_list:
         if len(a.childNodes):
@@ -55,14 +56,24 @@ def task_from_xml(task,xmlnode) :
             content = xml.dom.minidom.parseString(tas)
             cur_task.set_text(content.firstChild.toxml()) #pylint: disable-msg=E1103 
     cur_task.set_due_date(dates.strtodate(cleanxml.readTextNode(xmlnode,"duedate")))
-    cur_task.set_modified(cleanxml.readTextNode(xmlnode,"modified"))
     cur_task.set_start_date(dates.strtodate(cleanxml.readTextNode(xmlnode,"startdate")))
     cur_tags = xmlnode.getAttribute("tags").replace(' ','').split(",")
     if "" in cur_tags: cur_tags.remove("")
     for tag in cur_tags: cur_task.tag_added(saxutils.unescape(tag))
-    #Why should we sync here ? It makes no sense
-    #cur_task.sync()
-    
+
+    #REMOTE TASK IDS
+    remote_ids_list = xmlnode.getElementsByTagName("task-remote-ids")
+    for remote_id in remote_ids_list:
+        if remote_id.childNodes:
+            node = remote_id.childNodes[0]
+            backend_id = node.firstChild.nodeValue
+            remote_task_id = node.childNodes[1].firstChild.nodeValue
+            task.add_remote_id(backend_id, remote_task_id)
+    modified_string = cleanxml.readTextNode(xmlnode,"modified")
+    if modified_string:
+        modified_datetime = datetime.datetime.strptime(modified_string,\
+                                                    "%Y-%m-%dT%H:%M:%S")
+        cur_task.set_modified(modified_datetime)
     return cur_task
 
 #Task as parameter the doc where to put the XML node
@@ -77,10 +88,10 @@ def task_to_xml(doc,task) :
     t_xml.setAttribute("tags", tags_str[:-1])
     cleanxml.addTextNode(doc,t_xml,"title",task.get_title())
     cleanxml.addTextNode(doc,t_xml,"duedate", task.get_due_date().xml_str())
-    cleanxml.addTextNode(doc,t_xml,"modified",task.get_modified())
+    cleanxml.addTextNode(doc,t_xml,"modified",task.get_modified_string())
     cleanxml.addTextNode(doc,t_xml,"startdate", task.get_start_date().xml_str())
     cleanxml.addTextNode(doc,t_xml,"donedate", task.get_closed_date().xml_str())
-    childs = task.get_subtask_tids()
+    childs = task.get_children()
     for c in childs :
         cleanxml.addTextNode(doc,t_xml,"subtask",c)
     for a in task.attributes:
@@ -101,4 +112,18 @@ def task_to_xml(doc,task) :
         #t_xml.appendChild(element.firstChild)
         cleanxml.addTextNode(doc,t_xml,"content",desc)
     #self.__write_textnode(doc,t_xml,"content",t.get_text())
+
+    #REMOTE TASK IDS
+    remote_ids_element = doc.createElement("task-remote-ids")
+    t_xml.appendChild(remote_ids_element)
+    remote_ids_dict = task.get_remote_ids()
+    for backend_id, task_id in remote_ids_dict.iteritems():
+        backend_element = doc.createElement('backend')
+        remote_ids_element.appendChild(backend_element)
+        backend_element.appendChild(doc.createTextNode(backend_id))
+        task_element = doc.createElement('task-id')
+        backend_element.appendChild(task_element)
+        task_element.appendChild(doc.createTextNode(task_id))
+
+
     return t_xml
